package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.io.IoUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.deepoove.poi.plugin.table.LoopRowTableRenderPolicy;
import digital.laboratory.platform.common.feign.RemoteWord2PDFService;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.reagent.entity.ReagentConsumables;
import digital.laboratory.platform.reagent.entity.ReferenceMaterial;
import digital.laboratory.platform.reagent.mapper.ReferenceMaterialMapper;
import digital.laboratory.platform.reagent.service.ReagentConsumablesService;
import digital.laboratory.platform.reagent.service.ReferenceMaterialService;
import digital.laboratory.platform.reagent.vo.ReferenceMaterialVO;
import digital.laboratory.platform.reagent.vo.StandardSolutionCurveVO;
import feign.Response;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * 服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-23
 * @describe  服务实现类
 */

@Service
@SuppressWarnings("all")
public class ReferenceMaterialServiceImpl extends ServiceImpl<ReferenceMaterialMapper, ReferenceMaterial> implements ReferenceMaterialService {

    @Autowired
    private ReferenceMaterialService referenceMaterialService;

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;

    @Autowired
    private OssFile ossFile;

    @Autowired
    private RemoteWord2PDFService remoteWord2PDFService;

    @Override
    public List<ReferenceMaterialVO> getReferenceMaterialVOList(String batchDetailsId){

        LambdaQueryWrapper<ReferenceMaterial> referenceMaterialLambdaQueryWrapper = new LambdaQueryWrapper<>();

        referenceMaterialLambdaQueryWrapper.eq(ReferenceMaterial::getBatchDetailsId,batchDetailsId);

        List<ReferenceMaterial> list = referenceMaterialService.list(referenceMaterialLambdaQueryWrapper);

        List<ReferenceMaterialVO> referenceMaterialVOS = new ArrayList<>();

        for (ReferenceMaterial referenceMaterial : list) {

            ReagentConsumables byId = reagentConsumablesService.getById(referenceMaterial.getReagentConsumableId());

            ReferenceMaterialVO referenceMaterialVO = new ReferenceMaterialVO();

            BeanUtils.copyProperties(referenceMaterial,referenceMaterialVO);

            referenceMaterialVO.setReferenceMaterialName(byId.getReagentConsumableName());

            referenceMaterialVOS.add(referenceMaterialVO);
        }

        return referenceMaterialVOS;
    }

    @Override
    public void standardSolutionCurveTablePDF(List<StandardSolutionCurveVO> voList, String id, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) throws Exception {
        System.out.println("standardSolutionCurveTablePDF.................");

        Integer x = 1;
        for (StandardSolutionCurveVO standardSolutionCurveVO : voList) {

            standardSolutionCurveVO.setSerialnNumber(x.toString());

            x++;
        }

        String applyFileName = "标准工作曲线溶液配制记录表-" + id;
        //-----------------------------
        // 生成 word 版本的 验收记录表

        ByteArrayOutputStream bos = new ByteArrayOutputStream();

        ossFile.fileGet("template" + "/" + "标准工作曲线溶液配制记录表模板.docx", bos);

        byte[] templateArray = bos.toByteArray();

        ByteArrayInputStream bis = new ByteArrayInputStream(templateArray);

        bos.close();

        LoopRowTableRenderPolicy policy = new LoopRowTableRenderPolicy();

        ConfigureBuilder builder = Configure.builder().buildGramer("${", "}").useSpringEL(false);

        builder.bind("voList", policy).build();

        /**
         * 循环表格
         */
        XWPFTemplate template = XWPFTemplate.compile(bis, builder.build()).render(
                new HashMap<String, Object>() {{
                    put("voList",voList);
                    put("id",id);
                    put("size",1);


                }}
        );
        bis.close();

        ByteArrayOutputStream fosWord = new ByteArrayOutputStream();
        template.write(fosWord);
        template.close();

        //------------
        ByteArrayInputStream fisWord = new ByteArrayInputStream(fosWord.toByteArray());
        fosWord.close();

        //MockMultipartFile mockMultipartFile = new MockMultipartFile("file", entrustmentLetterFileName + ".docx", "application/vnd.openxmlformats-officedocument.wordprocessingml.document", fisWord);
        MockMultipartFile mockMultipartFile = new MockMultipartFile("file", applyFileName + ".docx", "image/jpg", fisWord);
        Response response = remoteWord2PDFService.word2pdf(mockMultipartFile);
        fisWord.close();


        ByteArrayOutputStream outPDF = new ByteArrayOutputStream();
        IoUtil.copy(response.body().asInputStream(), outPDF, IoUtil.DEFAULT_MIDDLE_BUFFER_SIZE);
        ByteArrayInputStream isPDF = new ByteArrayInputStream(outPDF.toByteArray());
        outPDF.close();


        ossFile.fileSave("document" + "/" + "standardSolutionCurve" + "/" + id + "/" + applyFileName + ".pdf", isPDF);
        isPDF.close();


        System.out.println(String.format("转换为 PDF 结束"));

    }

}
